import json

import pytest
from django_scopes import scope

from pretalx.api.serializers.speaker import (
    SpeakerOrgaSerializer,
    SpeakerSerializer,
    SubmitterSerializer,
)


@pytest.mark.django_db
def test_submitter_serializer(submission):
    with scope(event=submission.event):
        user = submission.speakers.first()
        data = SubmitterSerializer(user, event=submission.event).data
    assert data.keys() == {"name", "code", "biography", "avatar"}
    assert data["name"] == user.name
    assert data["code"] == user.code


@pytest.mark.django_db
def test_submitter_serializer_without_profile(submission):
    with scope(event=submission.event):
        user = submission.speakers.first()
        user.profiles.all().delete()
        data = SubmitterSerializer(user, event=submission.event).data
    assert data.keys() == {"name", "code", "biography", "avatar"}
    assert data["name"] == user.name
    assert data["code"] == user.code
    assert data["biography"] == ""


@pytest.mark.django_db
def test_speaker_serializer(slot):
    with scope(event=slot.submission.event):
        user_profile = slot.submission.speakers.first().profiles.first()
        user = user_profile.user
        data = SpeakerSerializer(user_profile).data
        assert slot.submission.code in data["submissions"]
    assert data.keys() == {"name", "code", "biography", "submissions", "avatar"}
    assert data["name"] == user.name
    assert data["code"] == user.code


@pytest.mark.django_db
def test_speaker_orga_serializer(slot):
    with scope(event=slot.submission.event):
        user_profile = slot.submission.speakers.first().profiles.first()
        user = user_profile.user
        data = SpeakerOrgaSerializer(user_profile).data
    assert data.keys() == {
        "name",
        "code",
        "biography",
        "submissions",
        "avatar",
        "answers",
        "email",
        "availabilities",
    }
    assert data["name"] == user.name
    assert data["code"] == user.code
    assert data["email"] == user.email
    assert slot.submission.code in data["submissions"]


@pytest.mark.django_db
@pytest.mark.parametrize(
    "is_visible_to_reviewers,is_reviewer,length",
    (
        (True, True, 1),
        (False, True, 0),
        (True, False, 1),
        (False, False, 1),
    ),
)
def test_speaker_answer_is_visible_to_reviewers(
    orga_client,
    review_user,
    submission,
    speaker,
    personal_answer,
    event,
    is_visible_to_reviewers,
    is_reviewer,
    length,
):
    if is_reviewer:
        orga_client.force_login(review_user)

    with scope(event=event):
        question = personal_answer.question
        question.is_visible_to_reviewers = is_visible_to_reviewers
        question.save()

    response = orga_client.get(
        submission.event.api_urls.speakers + "?questions=all", follow=True
    )
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 1
    assert content["results"][0]["name"] == speaker.name
    assert len(content["results"][0]["answers"]) == length


@pytest.mark.django_db
def test_can_only_see_public_speakers(
    client,
    slot,
    accepted_submission,
    rejected_submission,
    submission,
    impersonal_answer,
    event,
):
    response = client.get(submission.event.api_urls.speakers, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 1
    with scope(event=event):
        speaker = accepted_submission.speakers.first()
        assert content["results"][0]["name"] == speaker.name
        assert (
            content["results"][0]["biography"] == speaker.event_profile(event).biography
        )
    assert set(content["results"][0].keys()) == {
        "name",
        "code",
        "biography",
        "submissions",
        "avatar",
    }


@pytest.mark.django_db
def test_can_only_see_public_speakerss_if_public_schedule(
    client, slot, accepted_submission, rejected_submission, submission
):
    submission.event.feature_flags["show_schedule"] = False
    submission.event.save()
    response = client.get(submission.event.api_urls.speakers, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 0


@pytest.mark.django_db
def test_orga_can_see_all_speakers(
    orga_client,
    slot,
    accepted_submission,
    rejected_submission,
    submission,
    impersonal_answer,
):
    response = orga_client.get(
        submission.event.api_urls.speakers
        + f"?questions={impersonal_answer.question_id}",
        follow=True,
    )
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 2
    assert set(content["results"][0].keys()) == {
        "name",
        "code",
        "email",
        "biography",
        "submissions",
        "answers",
        "avatar",
        "availabilities",
    }
    assert set(content["results"][0]["answers"][0].keys()) == {
        "answer",
        "review",
        "answer_file",
        "person",
        "question",
        "submission",
        "options",
        "id",
    }


@pytest.mark.django_db
def test_orga_can_see_all_speakers_with_limit_and_offset(
    orga_client,
    slot,
    accepted_submission,
    rejected_submission,
    submission,
    impersonal_answer,
):
    response = orga_client.get(
        submission.event.api_urls.speakers + "?limit=1", follow=True
    )
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 2
    assert "offset=1" in content["next"]


@pytest.mark.django_db
def test_reviewer_cannot_see_speakers(
    review_client,
    slot,
    accepted_submission,
    rejected_submission,
    submission,
    impersonal_answer,
    event,
):
    with scope(event=event):
        submission.event.active_review_phase.can_see_speaker_names = False
        submission.event.active_review_phase.save()
    response = review_client.get(submission.event.api_urls.speakers, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert (
        content["count"] == 1
    )  # can see the slot's speaker, but not the other submissions'


@pytest.mark.django_db
def test_orga_can_see_all_speakers_even_nonpublic(
    orga_client, slot, accepted_submission, rejected_submission, submission
):
    submission.event.feature_flags["show_schedule"] = False
    submission.event.save()
    response = orga_client.get(submission.event.api_urls.speakers, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 2


@pytest.mark.django_db
def test_orga_speakers_with_multiple_talks_are_not_duplicated(
    client, speaker, slot, other_slot, accepted_submission, other_accepted_submission
):
    other_accepted_submission.speakers.add(accepted_submission.speakers.first())
    response = client.get(accepted_submission.event.api_urls.speakers, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 2



import rules

from pretalx.person.permissions import can_change_submissions, is_reviewer
from pretalx.submission.models import SubmissionStates
from pretalx.submission.permissions import can_be_edited


@rules.predicate
def is_event_visible(user, event):
    return event and event.is_public


@rules.predicate
def can_request_speakers(user, submission):
    # Only allow to request speakers if the field is active in the CfP
    return (
        submission.state != SubmissionStates.DRAFT
        and submission.event.cfp.request_additional_speaker
    )


rules.add_perm(
    "cfp.view_event", is_event_visible | (can_change_submissions | is_reviewer)
)
rules.add_perm("cfp.add_speakers", can_be_edited & can_request_speakers)



from io import BytesIO
from pathlib import Path

from django.conf import settings
from django.contrib.auth.password_validation import validate_password
from django.core.files import File
from django.core.files.uploadedfile import UploadedFile
from django.forms import CharField, FileField, ValidationError
from django.utils.translation import gettext_lazy as _
from PIL import Image

from pretalx.common.forms.widgets import (
    ClearableBasenameFileInput,
    ImageInput,
    PasswordConfirmationInput,
    PasswordStrengthInput,
)
from pretalx.common.templatetags.filesize import filesize

IMAGE_EXTENSIONS = (".png", ".jpg", ".gif", ".jpeg", ".svg")


class GlobalValidator:
    def __call__(self, value):
        return validate_password(value)


class PasswordField(CharField):
    default_validators = [GlobalValidator()]

    def __init__(self, *args, **kwargs):
        kwargs["widget"] = kwargs.get(
            "widget", PasswordStrengthInput(render_value=False)
        )
        super().__init__(*args, **kwargs)


class PasswordConfirmationField(CharField):
    def __init__(self, *args, **kwargs):
        kwargs["widget"] = kwargs.get(
            "widget",
            PasswordConfirmationInput(confirm_with=kwargs.pop("confirm_with", None)),
        )
        super().__init__(*args, **kwargs)


class SizeFileInput:
    """Takes the intended maximum upload size in bytes."""

    def __init__(self, *args, **kwargs):
        if "max_size" not in kwargs:  # Allow None, but only explicitly
            self.max_size = settings.FILE_UPLOAD_DEFAULT_LIMIT
        else:
            self.max_size = kwargs.pop("max_size")
        super().__init__(*args, **kwargs)
        self.size_warning = _("Please do not upload files larger than {size}!").format(
            size=filesize(self.max_size)
        )
        self.original_help_text = (
            getattr(self, "original_help_text", "") or self.help_text
        )
        self.added_help_text = getattr(self, "added_help_text", "") + self.size_warning
        self.help_text = self.original_help_text + " " + self.added_help_text
        self.widget.attrs["data-maxsize"] = self.max_size
        self.widget.attrs["data-sizewarning"] = self.size_warning

    def validate(self, value):
        super().validate(value)
        if (
            self.max_size
            and isinstance(value, UploadedFile)
            and value.size > self.max_size
        ):
            raise ValidationError(self.size_warning)


class ExtensionFileInput:
    widget = ClearableBasenameFileInput

    def __init__(self, *args, **kwargs):
        extensions = kwargs.pop("extensions")
        self.extensions = [i.lower() for i in extensions]
        super().__init__(*args, **kwargs)

    def validate(self, value):
        super().validate(value)
        if value:
            filename = value.name
            extension = Path(filename).suffix.lower()
            if extension not in self.extensions:
                raise ValidationError(
                    _(
                        "This filetype ({extension}) is not allowed, it has to be one of the following: "
                    ).format(extension=extension)
                    + ", ".join(self.extensions)
                )


class SizeFileField(SizeFileInput, FileField):
    pass


class ExtensionFileField(ExtensionFileInput, SizeFileInput, FileField):
    pass


class ImageField(ExtensionFileInput, SizeFileInput, FileField):
    # TODO: add better image field widget
    widget = ImageInput

    def __init__(self, *args, **kwargs):
        self.max_width = (
            kwargs.pop("max_width", None) or settings.IMAGE_DEFAULT_MAX_WIDTH
        )
        self.max_height = (
            kwargs.pop("max_height", None) or settings.IMAGE_DEFAULT_MAX_HEIGHT
        )
        super().__init__(*args, extensions=IMAGE_EXTENSIONS, **kwargs)

    def to_python(self, data):
        """Check that the file-upload field data contains a valid image (GIF,
        JPG, PNG, etc. -- whatever Pillow supports).

        Vendored from django.forms.fields.ImageField to add EXIF data
        removal. Can't use super() because we need to patch in the
        .png.fp object for some unholy (and possibly buggy) reason.
        """
        f = super().to_python(data)
        if f is None or f.name.endswith(".svg"):
            return f

        # We need to get a file object for Pillow. We might have a path or we might
        # have to read the data into memory.
        if hasattr(data, "temporary_file_path"):
            with open(data.temporary_file_path(), "rb") as temp_fp:
                file = BytesIO(temp_fp.read())
        else:
            if hasattr(data, "read"):
                file = BytesIO(data.read())
            else:
                file = BytesIO(data["content"])

        try:
            # load() could spot a truncated JPEG, but it loads the entire
            # image in memory, which is a DoS vector. See #3848 and #18520.
            image = Image.open(file)
            # verify() must be called immediately after the constructor.
            image.verify()

            # Annotating so subclasses can reuse it for their own validation
            f.image = image
            # Pillow doesn't detect the MIME type of all formats. In those
            # cases, content_type will be None.
            f.content_type = Image.MIME.get(image.format)
        except Exception as exc:
            # Pillow doesn't recognize it as an image.
            raise ValidationError(
                _(
                    "Upload a valid image. The file you uploaded was either not an "
                    "image or a corrupted image."
                )
            ) from exc
        if hasattr(f, "seek") and callable(f.seek):
            f.seek(0)

        image.fp = file
        if hasattr(image, "png"):  # Yeah, idk what's up with this
            image.png.fp = file

        stream = BytesIO()

        extension = ".jpg"
        if image.mode.lower() in ("rgba", "la", "pa"):
            extension = ".png"
        elif image.mode != "RGB":
            image = image.convert("RGB")

        stream.name = Path(data.name).stem + extension
        image_data = image.getdata()
        image_without_exif = Image.new(image.mode, image.size)
        image_without_exif.putdata(image_data)
        if self.max_height and self.max_width:
            image_without_exif.thumbnail((self.max_width, self.max_height))
        image_without_exif.save(
            stream, quality="web_high" if extension == ".jpg" else 95
        )
        stream.seek(0)
        return File(stream, name=data.name)



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from django.test import override_settings

from pretalx.common.css import validate_css
from pretalx.event.models import Event


@pytest.fixture
def valid_css():
    return """
body {
  background-color: #000;
  display: none;
}
.some-descriptor {
  /* Dotted borders are IN! */
  border-style: dotted dashed solid double;  /* more commenting */
  BORDER-color: red green blue yellow;
}
#best-descriptor {
  border: 5px solid red;
}
@media print {
  #best-descriptor {
    border: 5px solid blue;
  }
}
"""


@pytest.fixture
def invalid_css(valid_css):
    return (
        valid_css
        + """
a.other-descriptor {
    content: url("https://malicious.site.com");
}
"""
    )


def test_valid_css(valid_css):
    assert validate_css(valid_css) == valid_css


def test_invalid_css(invalid_css):
    with pytest.raises(ValidationError):
        validate_css(invalid_css)


@pytest.mark.django_db
@override_settings(COMPRESS_PRECOMPILERS=settings.COMPRESS_PRECOMPILERS_ORIGINAL)
def test_regenerate_css(event):
    from pretalx.common.tasks import regenerate_css

    event.primary_color = "#00ff00"
    event.settings.widget_css_checksum = "placeholder"
    event.save()
    regenerate_css(event.pk)
    event = Event.objects.get(pk=event.pk)
    for local_app in ["agenda", "cfp"]:
        assert event.settings.get(f"{local_app}_css_file")
        assert event.settings.get(f"{local_app}_css_checksum")
    assert event.settings.widget_css_checksum != "placeholder"
    assert event.settings.widget_css


@pytest.mark.django_db
def test_regenerate_css_no_color(event):
    from pretalx.common.tasks import regenerate_css

    event.primary_color = None
    event.save()
    regenerate_css(event.pk)
    event = Event.objects.get(pk=event.pk)
    for local_app in ["agenda", "cfp"]:
        assert not event.settings.get(f"{local_app}_css_file")
        assert not event.settings.get(f"{local_app}_css_checksum")
    assert not event.settings.widget_css


@pytest.mark.django_db
def test_regenerate_css_no_event():
    from pretalx.common.tasks import regenerate_css

    regenerate_css(123)



import pytest
from django.test import Client
from rest_framework.authtoken.models import Token


@pytest.mark.flaky(reruns=3)
@pytest.mark.django_db
def test_can_see_schedule_with_bearer_token(event, schedule, slot, orga_user):
    Token.objects.create(user=orga_user)
    client = Client(headers={"authorization": "Token " + orga_user.auth_token.key})
    event.feature_flags["show_schedule"] = False
    event.save()
    response = client.get(f"/{event.slug}/schedule.xml")
    assert response.status_code == 200
    assert slot.submission.title in response.content.decode()


@pytest.mark.flaky(reruns=3)
@pytest.mark.django_db
def test_cannot_see_schedule_with_wrong_bearer_token(event, schedule, slot, orga_user):
    Token.objects.create(user=orga_user)
    client = Client(
        headers={"authorization": "Token " + orga_user.auth_token.key + "xxx"}
    )
    event.feature_flags["show_schedule"] = False
    event.save()
    response = client.get(f"/{event.slug}/schedule.xml")
    assert response.status_code == 404
    assert slot.submission.title not in response.content.decode()



import logging

from csp.decorators import csp_update
from django.contrib import messages
from django.db import transaction
from django.http import Http404
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.crypto import get_random_string
from django.utils.decorators import method_decorator
from django.views import View

from pretalx.cfp.views.event import EventPageMixin
from pretalx.common.exceptions import SendMailException
from pretalx.common.phrases import phrases


class SubmitStartView(EventPageMixin, View):
    @staticmethod
    def get(request, *args, **kwargs):
        url = reverse(
            "cfp:event.submit",
            kwargs={
                "event": request.event.slug,
                "step": list(request.event.cfp_flow.steps_dict.keys())[0],
                "tmpid": get_random_string(length=6),
            },
        )
        if request.GET:
            url += f"?{request.GET.urlencode()}"
        return redirect(url)


@method_decorator(csp_update(IMG_SRC="https://www.gravatar.com"), name="dispatch")
class SubmitWizard(EventPageMixin, View):
    @transaction.atomic
    def dispatch(self, request, *args, **kwargs):
        self.event = self.request.event
        request.access_code = None
        if "access_code" in request.GET:
            access_code = request.event.submitter_access_codes.filter(
                code__iexact=request.GET["access_code"]
            ).first()
            if access_code and access_code.is_valid:
                request.access_code = access_code
        if not request.event.cfp.is_open and not request.access_code:
            messages.error(request, phrases.cfp.submissions_closed)
            return redirect(
                reverse("cfp:event.start", kwargs={"event": request.event.slug})
            )
        for step in request.event.cfp_flow.steps:
            if not step.is_applicable(request):
                continue
            if step.identifier == kwargs["step"]:
                break
            step.is_before = True
            step.resolved_url = step.get_step_url(request)
        if getattr(step, "is_before", False):  # The current step URL is incorrect
            raise Http404()
        handler = getattr(step, request.method.lower(), self.http_method_not_allowed)
        result = handler(request)

        if request.method == "POST" and request.POST.get("action", "submit") == "draft":
            return self.done(
                request,
                draft=True,
                steps=[
                    step
                    for step in request.event.cfp_flow.steps
                    if getattr(step, "is_before", False)
                    or step.identifier == kwargs["step"]
                    or step.identifier == "user"
                ],
            )
        if request.method == "GET" or (
            step.get_next_applicable(request) or not step.is_completed(request)
        ):
            return result
        return self.done(request)

    def done(self, request, draft=False, steps=None):
        # We are done, or at least we finished the last step. Time to check results.
        valid_steps = []
        steps = steps or request.event.cfp_flow.steps
        for step in steps:
            if step.is_applicable(request):
                if not step.is_completed(request):
                    query = {"draft": 1} if draft else None
                    return redirect(step.get_step_url(request, query=query))
                valid_steps.append(step)

        # We are done, or at least the data checks out. Time to save results.
        request.event.cfp_flow.steps_dict["user"].done(request)
        for step in valid_steps:
            if not step.identifier == "user":
                step.done(request, draft=draft)

        if not draft:
            try:
                request.submission.send_initial_mails(person=request.user)
            except SendMailException as exception:
                logging.getLogger("").warning(str(exception))
                messages.warning(request, phrases.cfp.submission_email_fail)

        return redirect(
            reverse("cfp:event.user.submissions", kwargs={"event": request.event.slug})
        )



import datetime as dt
import json
from zoneinfo import ZoneInfo

import pytest
from django.forms import ModelForm, ValidationError
from django.utils import timezone
from django_scopes import scope

from pretalx.person.models import SpeakerProfile
from pretalx.schedule.forms import AvailabilitiesFormMixin
from pretalx.schedule.models import Availability, Room

timezone.activate(ZoneInfo("UTC"))


class AvailabilitiesForm(AvailabilitiesFormMixin, ModelForm):
    class Meta:
        model = Room
        fields = []


@pytest.fixture
def availabilitiesform(event):
    event.date_from = dt.date(2017, 1, 1)
    event.date_to = dt.date(2017, 1, 2)

    return AvailabilitiesForm(
        event=event,
        instance=None,
    )


@pytest.mark.django_db
@pytest.mark.parametrize(
    "json,error",
    (
        ("{{{", "not valid json"),  # invalid json
        ("[]", "format"),  # not a dict
        ("42", "format"),  # not a dict
        ("{}", "format"),  # no "availabilities"
        ('{"availabilities": {}}', "format"),  # availabilities not a list
    ),
)
def test_parse_availabilities_json_fail(availabilitiesform, json, error):
    with pytest.raises(ValidationError) as excinfo:
        availabilitiesform._parse_availabilities_json(json)

    assert error in str(excinfo.value)


@pytest.mark.django_db
@pytest.mark.parametrize(
    "json",
    (
        ('{"availabilities": []}'),
        ('{"availabilities": [1]}'),
    ),
)
def test_parse_availabilities_json_success(availabilitiesform, json):
    availabilitiesform._parse_availabilities_json(json)


@pytest.mark.django_db
@pytest.mark.parametrize(
    "avail",
    (
        ([]),  # not a dict
        (42),  # not a dict
        ({}),  # missing attributes
        ({"start": True}),  # missing attributes
        ({"end": True}),  # missing attributes
        ({"start": True, "end": True, "foo": True}),  # extra attributes
    ),
)
def test_validate_availability_fail_format(availabilitiesform, avail):
    with pytest.raises(ValidationError) as excinfo:
        availabilitiesform._validate_availability(avail)

    assert "format" in str(excinfo.value)


@pytest.mark.django_db
@pytest.mark.parametrize(
    "avail",
    (
        ({"start": True, "end": True}),  # wrong type
        ({"start": "", "end": ""}),  # empty
        ({"start": "2017", "end": "2017"}),  # missing month
    ),
)
def test_validate_availability_fail_date(availabilitiesform, avail):
    with pytest.raises(ValidationError) as excinfo:
        availabilitiesform._validate_availability(avail)

    assert "invalid date" in str(excinfo.value)


@pytest.mark.django_db
@pytest.mark.parametrize(
    "avail",
    (
        ({"start": "2017-01-01 10:00:00", "end": "2017-01-01 12:00:00"}),  # same day
        ({"start": "2017-01-01 10:00:00", "end": "2017-01-02 12:00:00"}),  # next day
        (
            {"start": "2017-01-01 00:00:00", "end": "2017-01-02 00:00:00"}
        ),  # all day start
        ({"start": "2017-01-02 00:00:00", "end": "2017-01-03 00:00:00"}),  # all day end
    ),
)
def test_validate_availability_success(availabilitiesform, avail):
    availabilitiesform._validate_availability(avail)


@pytest.mark.django_db
@pytest.mark.parametrize(
    "avail",
    (
        (
            {"start": "2017-01-01 00:00:00", "end": "2017-01-01 08:00:00"}
        ),  # local time, start
        (
            {"start": "2017-01-02 05:00:00", "end": "2017-01-03 00:00:00"}
        ),  # local time, end
        (
            {"start": "2017-01-01 00:00:00-05:00", "end": "2017-01-01 00:00:00-05:00"}
        ),  # explicit timezone, start
        (
            {"start": "2017-01-02 05:00:00-05:00", "end": "2017-01-03 00:00:00-05:00"}
        ),  # explicit timezone, end
        (
            {"start": "2017-01-01 05:00:00+00:00", "end": "2017-01-01 00:00:00-05:00"}
        ),  # UTC, start
        (
            {"start": "2017-01-02 05:00:00-00:00", "end": "2017-01-03 05:00:00-00:00"}
        ),  # UTC, end
    ),
)
def test_validate_availability_tz_success(availabilitiesform, avail):
    availabilitiesform.event.timezone = "America/New_York"
    availabilitiesform.event.save()
    availabilitiesform._validate_availability(avail)


@pytest.mark.django_db
def test_validate_availability_daylightsaving(availabilitiesform):
    # https://github.com/pretalx/pretalx/issues/460
    availabilitiesform.event.timezone = "Europe/Berlin"
    availabilitiesform.event.date_from = dt.date(2018, 10, 22)
    availabilitiesform.event.date_to = dt.date(2018, 10, 28)
    availabilitiesform.event.save()
    availabilitiesform._validate_availability(
        {"start": "2018-10-22 00:00:00", "end": "2018-10-29 00:00:00"}
    )


@pytest.mark.django_db
@pytest.mark.parametrize(
    "strdate,expected",
    (
        ("2017-01-01 10:00:00", dt.datetime(2017, 1, 1, 10)),
        ("2017-01-01 10:00:00-05:00", dt.datetime(2017, 1, 1, 10)),
        ("2017-01-01 10:00:00-04:00", dt.datetime(2017, 1, 1, 9)),
    ),
)
def test_parse_datetime(availabilitiesform, strdate, expected):
    availabilitiesform.event.timezone = "America/New_York"
    availabilitiesform.event.save()
    del availabilitiesform.event.tz

    assert availabilitiesform.event.tz == ZoneInfo("America/New_York")
    expected = expected.replace(tzinfo=ZoneInfo("America/New_York"))
    actual = availabilitiesform._parse_datetime(strdate)
    assert actual == expected


@pytest.mark.django_db
@pytest.mark.parametrize(
    "json,error",
    (
        ("{{", "not valid json"),
        ('{"availabilities": [1]}', "format"),
    ),
)
def test_clean_availabilities_fail(availabilitiesform, json, error):
    with pytest.raises(ValidationError) as excinfo:
        availabilitiesform.cleaned_data = {"availabilities": json}
        availabilitiesform.clean_availabilities()

    assert error in str(excinfo.value)


@pytest.mark.django_db
@pytest.mark.parametrize(
    "json,expected",
    (
        ('{"availabilities": []}', []),
        (
            '{"availabilities": [{"start": "2017-01-01 10:00:00", "end": "2017-01-01 12:00:00"},'
            '{"start": "2017-01-02 11:00:00", "end": "2017-01-02 13:00:00"}]}',
            [
                Availability(
                    start=dt.datetime(2017, 1, 1, 10), end=dt.datetime(2017, 1, 1, 12)
                ),
                Availability(
                    start=dt.datetime(2017, 1, 2, 11), end=dt.datetime(2017, 1, 2, 13)
                ),
            ],
        ),
    ),
)
def test_clean_availabilities_success(availabilitiesform, json, expected):
    availabilitiesform.cleaned_data = {"availabilities": json}
    actual = availabilitiesform.clean_availabilities()

    assert len(actual) == len(expected)

    for act, exp in zip(actual, expected):
        assert act.start.replace(tzinfo=None) == exp.start
        assert act.end.replace(tzinfo=None) == exp.end
        assert act.event_id == availabilitiesform.event.id
        assert act.id is None


@pytest.mark.django_db
@pytest.mark.parametrize(
    "instancegen,fk_name",
    (
        (lambda event_id: Room.objects.create(event_id=event_id), "room_id"),
        (
            lambda event_id: SpeakerProfile.objects.create(event_id=event_id),
            "person_id",
        ),
    ),
)
def test_set_foreignkeys(availabilitiesform, instancegen, fk_name):
    availabilities = [
        Availability(
            start=dt.datetime(2017, 1, 1, 10), end=dt.datetime(2017, 1, 1, 12)
        ),
        Availability(
            start=dt.datetime(2017, 1, 2, 10), end=dt.datetime(2017, 1, 2, 15)
        ),
    ]
    instance = instancegen(availabilitiesform.event.id)
    availabilitiesform._set_foreignkeys(instance, availabilities)

    for avail in availabilities:
        assert getattr(avail, fk_name) == instance.id


@pytest.mark.django_db
def test_replace_availabilities(availabilitiesform):
    with scope(event=availabilitiesform.event):
        instance = Room.objects.create(event_id=availabilitiesform.event.id)
        Availability.objects.bulk_create(
            [
                Availability(
                    room_id=instance.id,
                    event_id=availabilitiesform.event.id,
                    start=dt.datetime(2017, 1, 1, 10, tzinfo=ZoneInfo("UTC")),
                    end=dt.datetime(2017, 1, 1, 12, tzinfo=ZoneInfo("UTC")),
                ),
                Availability(
                    room_id=instance.id,
                    event_id=availabilitiesform.event.id,
                    start=dt.datetime(2017, 1, 2, 10, tzinfo=ZoneInfo("UTC")),
                    end=dt.datetime(2017, 1, 2, 15, tzinfo=ZoneInfo("UTC")),
                ),
            ]
        )

        expected = [
            Availability(
                room_id=instance.id,
                event_id=availabilitiesform.event.id,
                start=dt.datetime(2017, 1, 1, 12, tzinfo=ZoneInfo("UTC")),
                end=dt.datetime(2017, 1, 1, 12, tzinfo=ZoneInfo("UTC")),
            ),
            Availability(
                room_id=instance.id,
                event_id=availabilitiesform.event.id,
                start=dt.datetime(2017, 1, 2, 12, tzinfo=ZoneInfo("UTC")),
                end=dt.datetime(2017, 1, 2, 15, tzinfo=ZoneInfo("UTC")),
            ),
        ]

        availabilitiesform._replace_availabilities(instance, expected)

        actual = instance.availabilities.all()
        for act, exp in zip(actual, expected):
            assert act.start == exp.start


@pytest.mark.django_db
@pytest.mark.parametrize(
    "avail,expected",
    (
        (
            Availability(
                start=dt.datetime(2017, 1, 1, 10, tzinfo=ZoneInfo("UTC")),
                end=dt.datetime(2017, 1, 1, 12, tzinfo=ZoneInfo("UTC")),
            ),
            {
                "start": "2017-01-01T10:00:00Z",
                "end": "2017-01-01T12:00:00Z",
                "allDay": False,
            },
        ),
        (
            Availability(
                start=dt.datetime(2017, 1, 1, 10, tzinfo=ZoneInfo("UTC")),
                end=dt.datetime(2017, 1, 2, tzinfo=ZoneInfo("UTC")),
            ),
            {
                "start": "2017-01-01T10:00:00Z",
                "end": "2017-01-02T00:00:00Z",
                "allDay": False,
            },
        ),
        (
            Availability(
                start=dt.datetime(2017, 1, 1, tzinfo=ZoneInfo("UTC")),
                end=dt.datetime(2017, 1, 1, 10, tzinfo=ZoneInfo("UTC")),
            ),
            {
                "start": "2017-01-01T00:00:00Z",
                "end": "2017-01-01T10:00:00Z",
                "allDay": False,
            },
        ),
        (
            Availability(
                start=dt.datetime(2017, 1, 1, 10, tzinfo=ZoneInfo("UTC")),
                end=dt.datetime(2017, 1, 2, tzinfo=ZoneInfo("UTC")),
            ),
            {
                "start": "2017-01-01T10:00:00Z",
                "end": "2017-01-02T00:00:00Z",
                "allDay": False,
            },
        ),
        (
            Availability(
                start=dt.datetime(2017, 1, 1, tzinfo=ZoneInfo("UTC")),
                end=dt.datetime(2017, 1, 2, tzinfo=ZoneInfo("UTC")),
            ),
            {
                "start": "2017-01-01T00:00:00Z",
                "end": "2017-01-02T00:00:00Z",
                "allDay": True,
            },
        ),
    ),
)
def test_serialize_availability(availabilitiesform, avail, expected):
    with timezone.override(ZoneInfo("UTC")):
        actual = avail.serialize()
    del actual["id"]
    assert actual == expected


@pytest.mark.django_db
@pytest.mark.parametrize(
    "avails,expected,tzname",
    (
        (
            [
                Availability(
                    start=dt.datetime(2017, 1, 1, 10, tzinfo=ZoneInfo("UTC")),
                    end=dt.datetime(2017, 1, 1, 12, tzinfo=ZoneInfo("UTC")),
                )
            ],
            '{"availabilities": [{"id": 1, "start": "2017-01-01T10:00:00Z", "end": "2017-01-01T12:00:00Z", "allDay": false}], "event": {"timezone": "UTC", "date_from": "2017-01-01", "date_to": "2017-01-02"}}',
            "UTC",
        ),
        (
            [],
            '{"availabilities": [], "event": {"timezone": "UTC", "date_from": "2017-01-01", "date_to": "2017-01-02"}}',
            "UTC",
        ),
        (
            None,
            '{"availabilities": [], "event": {"timezone": "UTC", "date_from": "2017-01-01", "date_to": "2017-01-02"}}',
            "UTC",
        ),
        (
            None,
            '{"availabilities": [], "event": {"timezone": "America/New_York", "date_from": "2017-01-01", "date_to": "2017-01-02"}}',
            "America/New_York",
        ),
    ),
)
def test_serialize(availabilitiesform, avails, expected, tzname):
    with scope(event=availabilitiesform.event), timezone.override(ZoneInfo("UTC")):
        availabilitiesform.event.timezone = tzname
        availabilitiesform.event.save()

        if avails is not None:
            instance = Room.objects.create(event_id=availabilitiesform.event.id)
            for avail in avails:
                avail.event_id = availabilitiesform.event.id
                avail.room_id = instance.id
                avail.save()
        else:
            instance = None

        if avails:
            expected = json.loads(expected)
            for a, j in zip(avails, expected["availabilities"]):
                j["id"] = a.pk
            expected = json.dumps(expected)

        actual = availabilitiesform._serialize(availabilitiesform.event, instance)
        assert actual == expected


@pytest.mark.django_db
def test_chained(availabilitiesform, room):
    """make sure the Mixin can actually deserialize the data it serialized."""
    with scope(event=room.event):
        room.event.timezone = "America/New_York"
        room.event.save()
        del room.event.tz
        room.save()
        # normal
        Availability.objects.create(
            event=availabilitiesform.event,
            room=room,
            start=dt.datetime(2017, 1, 1, 10, tzinfo=room.event.tz),
            end=dt.datetime(2017, 1, 1, 12, tzinfo=room.event.tz),
        )
        # all day
        Availability.objects.create(
            event=availabilitiesform.event,
            room=room,
            start=dt.datetime(2017, 1, 1, tzinfo=room.event.tz),
            end=dt.datetime(2017, 1, 3, tzinfo=room.event.tz),
        )

        form = AvailabilitiesForm(
            event=availabilitiesform.event,
            instance=room,
        )

        form.cleaned_data = form.initial
        form.cleaned_data["availabilities"] = form.clean_availabilities()
        form.save()

        avails = Room.objects.first().availabilities.order_by("-start")
        # One of these is contained in the other!
        assert len(avails) == 1
        assert avails[0].start == dt.datetime(2017, 1, 1, 5, tzinfo=ZoneInfo("UTC"))
        assert avails[0].end == dt.datetime(2017, 1, 3, 5, tzinfo=ZoneInfo("UTC"))



import pytest
from django_scopes import scope

from pretalx.common.log_display import LOG_NAMES
from pretalx.common.models.log import ActivityLog


@pytest.fixture
def activity_log(event, submission):
    return ActivityLog(
        event=event, content_object=submission, action_type="pretalx.submission.create"
    )


@pytest.mark.django_db
def test_activity_log_display(activity_log):
    assert activity_log.display() == LOG_NAMES.get(activity_log.action_type)


@pytest.mark.django_db
def test_activity_log_display_incorrect(activity_log):
    activity_log.action_type = "foo"
    assert activity_log.display() == "foo"


@pytest.mark.django_db
def test_log_urls(activity_log, submission, choice_question, answer, mail_template):
    with scope(event=submission.event):
        assert activity_log.get_public_url() == submission.urls.public
        assert activity_log.get_orga_url() == submission.orga_urls.base

        activity_log.content_object = submission.event.cfp
        assert activity_log.get_public_url() == submission.event.cfp.urls.public
        assert activity_log.get_orga_url() == submission.event.cfp.urls.text

        activity_log.content_object = choice_question
        assert activity_log.get_public_url() == ""
        assert activity_log.get_orga_url() == choice_question.urls.base

        activity_log.content_object = choice_question.options.first()
        assert activity_log.get_orga_url() == choice_question.urls.base

        activity_log.content_object = answer
        assert activity_log.get_orga_url() == answer.submission.orga_urls.base
        answer.submission = None
        assert activity_log.get_orga_url() == answer.question.urls.base

        activity_log.content_object = mail_template
        assert activity_log.get_orga_url() == mail_template.urls.base



from django.dispatch import receiver

from pretalx.cfp.signals import footer_link, html_above_profile_page, html_head
from pretalx.common.signals import register_locales
from pretalx.orga.signals import (
    activate_event,
    nav_event,
    nav_event_settings,
    nav_global,
)


@receiver(register_locales)
def register_locales_test(sender, **kwargs):
    return [("testlocale", "Test")]


@receiver(footer_link)
def footer_link_test(sender, request, **kwargs):
    event = getattr(request, "event", None)
    link = f"/{event.slug}/test" if event else "/test"
    return [{"link": link, "label": "test"}]


@receiver(html_head)
def html_head_test(sender, request, **kwargs):
    if sender.slug != "ignore_signal":
        return '<meta property="pretalx:foo" content="bar">'


@receiver(html_above_profile_page)
def html_above_profile_page_test(sender, request, **kwargs):
    if sender.slug != "ignore_signal":
        return "<p></p>"


@receiver(nav_event_settings)
def nav_event_settings_test(sender, request, **kwargs):
    return []


@receiver(nav_event)
def nav_event_test(sender, request, **kwargs):
    return []


@receiver(nav_global)
def nav_global_test(sender, request, **kwargs):
    return {"label": "root", "url": "/"}


@receiver(activate_event)
def activate_event_test(sender, request, **kwargs):
    if sender.slug == "donottakelive":
        raise Exception("It's not safe to go alone take this")



from urllib.parse import urlparse

import pytest
from django.conf import settings


@pytest.fixture(autouse=True)
def env(event):
    settings.SITE_URL = "http://example.com:444"
    settings.SITE_NETLOC = urlparse(settings.SITE_URL).netloc


@pytest.mark.django_db
def test_event_on_unknown_domain(event, client):
    r = client.get("/{event.slug}/", HTTP_HOST="foobar")
    assert r.status_code == 404, r.content.decode()


@pytest.mark.django_db
def test_orga_event_on_unknown_domain(event, client):
    r = client.get("/orga/event/{event.slug}/", HTTP_HOST="foobar")
    assert r.status_code == 404, r.content.decode()



import json

import pytest

from pretalx.api.serializers.event import EventSerializer


@pytest.mark.django_db
def test_event_serializer(event):
    data = EventSerializer(event).data
    data["name"] = None
    assert data == {
        "name": None,
        "slug": event.slug,
        "is_public": event.is_public,
        "date_from": event.date_from.isoformat(),
        "date_to": event.date_to.isoformat(),
        "timezone": event.timezone,
        "urls": {
            "base": event.urls.base.full(),
            "schedule": event.urls.schedule.full(),
            "login": event.urls.login.full(),
            "feed": event.urls.feed.full(),
        },
    }


@pytest.mark.django_db
def test_can_only_see_public_events(client, event, other_event):
    other_event.is_public = False
    other_event.save()
    assert event.is_public
    assert not other_event.is_public

    response = client.get("/api/events", follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert len(content) == 1, content
    assert content[0]["name"]["en"] == event.name


@pytest.mark.django_db
def test_can_only_see_public_events_in_detail(client, event):
    assert event.is_public
    response = client.get(event.api_urls.base, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["name"]["en"] == event.name

    event.is_public = False
    event.save()

    response = client.get(event.api_urls.base, follow=True)
    assert response.status_code == 404
    assert event.name not in response.content.decode()


@pytest.mark.django_db
def test_orga_can_see_nonpublic_events(orga_client, event, other_event):
    event.is_public = False
    event.save()
    assert not event.is_public
    assert other_event.is_public

    response = orga_client.get("/api/events", follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert len(content) == 2, content
    assert content[-1]["name"]["en"] == event.name



import statistics
from collections import defaultdict
from contextlib import suppress

from django.contrib import messages
from django.db import transaction
from django.db.models import Count, Max, OuterRef, Q, Subquery
from django.forms.models import BaseModelFormSet, modelformset_factory
from django.shortcuts import get_object_or_404, redirect
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from django.views.generic import FormView, TemplateView
from django_context_decorator import context

from pretalx.common.mixins.views import EventPermissionRequired, PermissionRequired
from pretalx.common.views import CreateOrUpdateView
from pretalx.orga.forms.review import (
    DirectionForm,
    ProposalForReviewerForm,
    ReviewAssignImportForm,
    ReviewerForProposalForm,
    ReviewExportForm,
    ReviewForm,
    TagsForm,
)
from pretalx.orga.forms.submission import SubmissionStateChangeForm
from pretalx.orga.views.submission import BaseSubmissionList
from pretalx.person.models import User
from pretalx.submission.forms import QuestionsForm
from pretalx.submission.models import Review, Submission, SubmissionStates


class ReviewDashboard(EventPermissionRequired, BaseSubmissionList):
    template_name = "orga/review/dashboard.html"
    permission_required = "orga.view_review_dashboard"
    paginate_by = None
    usable_states = (
        SubmissionStates.SUBMITTED,
        SubmissionStates.ACCEPTED,
        SubmissionStates.REJECTED,
        SubmissionStates.CONFIRMED,
    )

    def filter_range(self, queryset):
        review_count = self.request.GET.get("review-count") or ","
        if "," not in review_count:
            return queryset
        min_reviews, max_reviews = review_count.split(",", maxsplit=1)
        if min_reviews:
            with suppress(Exception):
                min_reviews = int(min_reviews)
                if min_reviews > 0:
                    queryset = queryset.filter(review_count__gte=min_reviews)
        if max_reviews:
            with suppress(Exception):
                max_reviews = int(max_reviews)
                if max_reviews < self.max_review_count:
                    queryset = queryset.filter(review_count__lte=max_reviews)
        return queryset

    def get_queryset(self):
        aggregate_method = self.request.event.review_settings["aggregate_method"]
        # TODO remove mean fallback once we drop Python 3.7
        statistics_method = (
            statistics.median
            if aggregate_method == "median"
            else (statistics.fmean if hasattr(statistics, "fmean") else statistics.mean)
        )
        queryset = self._get_base_queryset(for_review=True).filter(
            state__in=self.usable_states
        )
        queryset = self.filter_queryset(queryset).annotate(
            review_count=Count("reviews", distinct=True),
            review_nonnull_count=Count(
                "reviews", distinct=True, filter=Q(reviews__score__isnull=False)
            ),
        )
        queryset = self.filter_range(queryset)

        user_reviews = Review.objects.filter(
            user=self.request.user, submission_id=OuterRef("pk")
        ).values("score")

        queryset = (
            queryset.annotate(user_score=Subquery(user_reviews))
            .select_related("track", "submission_type")
            .prefetch_related(
                "speakers",
                "reviews",
                "reviews__user",
                "reviews__scores",
                "tags",
                "answers",
            )
        )

        for submission in queryset:
            if self.can_see_all_reviews:
                submission.current_score = (
                    submission.median_score
                    if aggregate_method == "median"
                    else submission.mean_score
                )
                if (
                    self.independent_categories
                ):  # Assemble medians/means on the fly. Yay.
                    independent_ids = [cat.pk for cat in self.independent_categories]
                    mapping = defaultdict(list)
                    for review in submission.reviews.all():
                        for score in review.scores.all():
                            if score.category_id in independent_ids:
                                mapping[score.category_id].append(score.value)
                    mapping = {
                        key: round(statistics_method(value), 1)
                        for key, value in mapping.items()
                    }
                    result = []
                    for category in self.independent_categories:
                        result.append(mapping.get(category.pk))
                    submission.independent_scores = result
                if self.short_questions:
                    answers = {
                        answer.question_id: answer
                        for answer in submission.answers.all()
                    }
                    submission.short_answers = [
                        answers.get(
                            question.id,
                            {"question_id": question.id, "answer_string": ""},
                        )
                        for question in self.short_questions
                    ]
            else:
                reviews = [
                    review
                    for review in submission.reviews.all()
                    if review.user == self.request.user
                ]
                submission.current_score = None
                if reviews:
                    review = reviews[0]
                    submission.current_score = review.score
                    if self.independent_categories:
                        mapping = {s.category_id: s.value for s in review.scores.all()}
                        result = []
                        for category in self.independent_categories:
                            result.append(mapping.get(category.pk))
                        submission.independent_scores = result
                elif self.independent_categories:
                    submission.independent_scores = [
                        None for _ in range(len(self.independent_categories))
                    ]

        return self.sort_queryset(queryset)

    def sort_queryset(self, queryset):
        order_prevalence = {
            "default": ("is_assigned", "state", "current_score", "code"),
            "score": ("current_score", "state", "code"),
            "my_score": ("user_score", "current_score", "state", "code"),
            "count": ("review_nonnull_count", "code"),
        }
        ordering = self.request.GET.get("sort", "default")
        reverse = True
        if ordering.startswith("-"):
            reverse = False
            ordering = ordering[1:]

        order = order_prevalence.get(ordering, order_prevalence["default"])

        def get_order_tuple(obj):
            result = []
            for key in order:
                value = getattr(obj, key)
                if value is None:
                    value = 100 * -int(reverse or -1)
                result.append(value)
            return tuple(result)

        return sorted(
            queryset,
            key=get_order_tuple,
            reverse=reverse,
        )

    @context
    @cached_property
    def can_accept_submissions(self):
        return self.request.event.submissions.filter(
            state=SubmissionStates.SUBMITTED
        ).exists() and self.request.user.has_perm(
            "submission.accept_or_reject_submissions", self.request.event
        )

    @context
    @cached_property
    def can_see_all_reviews(self):
        return self.request.user.has_perm("orga.view_all_reviews", self.request.event)

    @context
    @cached_property
    def max_review_count(self):
        return (
            self.request.event.submissions.all()
            .annotate(review_count=Count("reviews", distinct=True))
            .aggregate(Max("review_count"))
            .get("review_count__max")
        )

    @context
    @cached_property
    def submissions_reviewed(self):
        return Review.objects.filter(
            user=self.request.user, submission__event=self.request.event
        ).values_list("submission_id", flat=True)

    @context
    @cached_property
    def show_submission_types(self):
        return self.request.event.submission_types.all().count() > 1

    @context
    @cached_property
    def short_questions(self):
        from pretalx.submission.models import QuestionVariant

        return self.request.event.questions.filter(
            target="submission",
            variant__in=[
                QuestionVariant.BOOLEAN,
                QuestionVariant.CHOICES,
                QuestionVariant.MULTIPLE,
                QuestionVariant.DATE,
                QuestionVariant.DATETIME,
                QuestionVariant.BOOLEAN,
                QuestionVariant.NUMBER,
            ],
        )

    @context
    @cached_property
    def independent_categories(self):
        return self.request.event.score_categories.all().filter(
            is_independent=True, active=True
        )

    @context
    @cached_property
    def show_tracks(self):
        return (
            self.request.event.feature_flags["use_tracks"]
            and self.request.event.tracks.all().count() > 1
        )

    def get_context_data(self, **kwargs):
        result = super().get_context_data(**kwargs)
        missing_reviews = Review.find_missing_reviews(
            self.request.event, self.request.user
        )
        result[
            "missing_reviews"
        ] = (
            missing_reviews.count()
        )  # Do NOT use len() here! It yields a different result.
        result["next_submission"] = missing_reviews[0] if missing_reviews else None
        return result

    def get_pending(self, request):
        form = SubmissionStateChangeForm(request.POST)
        if form.is_valid():
            return form.cleaned_data.get("pending")

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        total = {"accept": 0, "reject": 0, "error": 0}
        pending = self.get_pending(request)
        for key, value in request.POST.items():
            if not key.startswith("s-") or value not in ["accept", "reject"]:
                continue
            code = key.strip("s-")
            try:
                submission = request.event.submissions.filter(
                    state=SubmissionStates.SUBMITTED
                ).get(code=code)
            except (Submission.DoesNotExist, ValueError):
                total["error"] += 1
                continue
            if not request.user.has_perm(
                "submission." + value + "_submission", submission
            ):
                total["error"] += 1
                continue
            if pending:
                submission.pending_state = (
                    SubmissionStates.ACCEPTED
                    if value == "accept"
                    else SubmissionStates.REJECTED
                )
                submission.save()
            else:
                getattr(submission, value)(person=request.user)
            total[value] += 1
        if total["accept"] or total["reject"]:
            msg = str(
                _(
                    "Success! {accepted} proposals were accepted, {rejected} proposals were rejected."
                )
            ).format(accepted=total["accept"], rejected=total["reject"])
            if total["error"]:
                msg += " " + str(
                    _("We were unable to change the state of {count} proposals.")
                ).format(count=total["error"])
            messages.success(request, msg)
        else:
            messages.error(
                request,
                str(
                    _("We were unable to change the state of all {count} proposals.")
                ).format(count=total["error"]),
            )
        return super().get(request, *args, **kwargs)


class ReviewSubmission(PermissionRequired, CreateOrUpdateView):
    form_class = ReviewForm
    model = Review
    template_name = "orga/submission/review.html"
    permission_required = "submission.view_reviews"
    write_permission_required = "submission.review_submission"

    @context
    @cached_property
    def submission(self):
        return get_object_or_404(
            self.request.event.submissions.prefetch_related("speakers"),
            code__iexact=self.kwargs["code"],
        )

    @cached_property
    def object(self):
        review = (
            self.submission.reviews.exclude(user__in=self.submission.speakers.all())
            .filter(user=self.request.user)
            .first()
        )
        return review

    @context
    @cached_property
    def review_display(self):
        if self.object:
            review = self.object
            return {
                "score": review.display_score,
                "scores": self.get_scores_for_review(review),
                "text": review.text,
                "user": review.user.get_display_name(),
                "answers": [
                    review.answers.filter(question=question).first()
                    for question in self.qform.queryset
                ],
            }

    @context
    @cached_property
    def has_anonymised_review(self):
        return self.request.event.review_phases.filter(
            can_see_speaker_names=False
        ).exists()

    @context
    @cached_property
    def anonymise_review(self):
        return not getattr(
            self.request.event.active_review_phase, "can_see_speaker_names", True
        )

    def get_object(self):
        return self.object

    def get_permission_object(self):
        return self.submission

    @context
    @cached_property
    def read_only(self):
        if self.request.user in self.submission.speakers.all():
            return True
        if self.object and self.object.pk:
            return not self.request.user.has_perm(
                "submission.edit_review", self.get_object()
            )
        return not self.request.user.has_perm(
            "submission.review_submission", self.get_object() or self.submission
        )

    @context
    def profiles(self):
        return [
            speaker.event_profile(self.request.event)
            for speaker in self.submission.speakers.all()
        ]

    @context
    @cached_property
    def score_categories(self):
        return self.submission.score_categories

    def get_scores_for_review(self, review):
        scores = []
        score_format = self.request.event.review_settings.get(
            "score_format", "words_numbers"
        )
        review_scores = {score.category: score for score in review.scores.all()}
        for category in self.score_categories:
            score = review_scores.get(category)
            if score:
                scores.append(score.format(score_format))
            else:
                scores.append("×")
        return scores

    @context
    def reviews(self):
        return [
            {
                "score": review.display_score,
                "scores": self.get_scores_for_review(review),
                "text": review.text,
                "user": review.user.get_display_name(),
                "answers": [
                    review.answers.filter(question=question).first()
                    for question in self.qform.queryset
                ],
            }
            for review in self.submission.reviews.exclude(
                pk=(self.object.pk if self.object else None)
            ).prefetch_related("scores", "scores__category")
        ]

    @context
    @cached_property
    def qform(self):
        return QuestionsForm(
            target="reviewer",
            event=self.request.event,
            data=(self.request.POST if self.request.method == "POST" else None),
            files=(self.request.FILES if self.request.method == "POST" else None),
            speaker=self.request.user,
            review=self.object,
            readonly=self.read_only,
        )

    @context
    @cached_property
    def tags_form(self):
        if not self.request.event.tags.all().exists():
            return
        return TagsForm(
            event=self.request.event,
            instance=self.submission,
            data=(self.request.POST if self.request.method == "POST" else None),
            read_only=self.read_only,
        )

    def get_context_data(self, **kwargs):
        result = super().get_context_data(**kwargs)
        result["done"] = self.request.user.reviews.filter(
            submission__event=self.request.event
        ).count()
        result["total_reviews"] = (
            Review.find_missing_reviews(self.request.event, self.request.user).count()
            + result["done"]
        )
        if result["total_reviews"]:
            result["percentage"] = int(result["done"] * 100 / result["total_reviews"])
        return result

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["event"] = self.request.event
        kwargs["user"] = self.request.user
        kwargs["read_only"] = self.read_only
        kwargs["categories"] = self.score_categories
        return kwargs

    def form_valid(self, form):
        if not self.qform.is_valid():
            messages.error(self.request, _("There have been errors with your input."))
            return redirect(self.get_success_url())
        if self.tags_form and not self.tags_form.is_valid():
            messages.error(self.request, _("There have been errors with your input."))
            return redirect(self.get_success_url())
        form.instance.submission = self.submission
        form.instance.user = self.request.user
        form.save()
        self.qform.review = form.instance
        self.qform.save()
        if self.tags_form:
            self.tags_form.save()
        return super().form_valid(form)

    def post(self, request, *args, **kwargs):
        action = self.request.POST.get("review_submit") or "save"
        if action == "abstain":
            Review.objects.get_or_create(
                user=self.request.user, submission=self.submission
            )
            return redirect(self.get_success_url())
        if action == "skip_for_now":
            key = f"{self.request.event.slug}_ignored_reviews"
            ignored_submissions = self.request.session.get(key) or []
            ignored_submissions.append(self.submission.pk)
            self.request.session[key] = ignored_submissions
            return redirect(self.get_success_url())
        return super().post(request, *args, **kwargs)

    def get_success_url(self) -> str:
        action = self.request.POST.get("review_submit")
        if action == "save":
            return self.submission.orga_urls.reviews
            # save, skip_for_now, save_and_next

        key = f"{self.request.event.slug}_ignored_reviews"
        ignored_submissions = self.request.session.get(key) or []
        next_submission = Review.find_missing_reviews(
            self.request.event,
            self.request.user,
            ignore=ignored_submissions,
        ).first()
        if not next_submission:
            ignored_submissions = (
                [self.submission.pk] if action == "skip_for_now" else []
            )
            next_submission = Review.find_missing_reviews(
                self.request.event,
                self.request.user,
                ignore=ignored_submissions,
            ).first()
        self.request.session[key] = ignored_submissions
        if next_submission:
            return next_submission.orga_urls.reviews
        messages.success(self.request, _("Nice, you have no proposals left to review!"))
        return self.request.event.orga_urls.reviews


class ReviewSubmissionDelete(EventPermissionRequired, TemplateView):
    template_name = "orga/review/submission_delete.html"
    permission_required = "orga.remove_review"


class RegenerateDecisionMails(EventPermissionRequired, TemplateView):
    template_name = "orga/review/regenerate_decision_mails.html"
    permission_required = "orga.change_submissions"

    def get_queryset(self):
        return (
            self.request.event.submissions.filter(
                state__in=[SubmissionStates.ACCEPTED, SubmissionStates.REJECTED],
                speakers__isnull=False,
            )
            .prefetch_related("speakers")
            .distinct()
        )

    @context
    @cached_property
    def count(self):
        return sum(len(proposal.speakers.all()) for proposal in self.get_queryset())

    def post(self, request, **kwargs):
        for submission in self.get_queryset():
            submission.send_state_mail()
        messages.success(
            request,
            _("{count} emails were generated and placed in the outbox.").format(
                count=self.count
            ),
        )
        return redirect(self.request.event.orga_urls.reviews)


class ReviewAssignment(EventPermissionRequired, FormView):
    template_name = "orga/review/assignment.html"
    permission_required = "orga.change_settings"
    form_class = DirectionForm

    @cached_property
    def form_type(self):
        direction = self.request.GET.get("direction")
        if not direction or direction not in ("reviewer", "submission"):
            return "reviewer"
        return direction

    def get_form(self):
        return DirectionForm(self.request.GET)

    @context
    @cached_property
    def review_teams(self):
        return self.request.event.teams.filter(is_reviewer=True)

    @context
    @cached_property
    def formset(self):
        proposals = self.request.event.submissions.order_by("title")
        reviewers = (
            User.objects.filter(
                teams__in=self.request.event.teams.filter(is_reviewer=True)
            )
            .order_by("name")
            .distinct()
        )

        if self.form_type == "submission":
            formset_class = modelformset_factory(
                model=Submission,
                form=ReviewerForProposalForm,
                formset=BaseModelFormSet,
                can_delete=False,
                extra=0,
                max_num=0,
            )
            result = formset_class(
                self.request.POST if self.request.method == "POST" else None,
                files=self.request.FILES if self.request.method == "POST" else None,
                queryset=proposals,
                form_kwargs={"reviewers": reviewers},
                prefix="formset",
            )
            return result
        else:
            formset_class = modelformset_factory(
                User,
                form=ProposalForReviewerForm,
                formset=BaseModelFormSet,
                can_delete=False,
                extra=0,
                max_num=0,
            )
            return formset_class(
                self.request.POST if self.request.method == "POST" else None,
                files=self.request.FILES if self.request.method == "POST" else None,
                queryset=reviewers,
                form_kwargs={"proposals": proposals},
                prefix="formset",
            )

    def post(self, request, *args, **kwargs):
        if not self.formset.is_valid():
            return self.get(self.request, *self.args, **self.kwargs)

        for form in self.formset:
            form.save()
        messages.success(request, _("Saved!"))
        return self.get(self.request, *self.args, **self.kwargs)


class ReviewAssignmentImport(EventPermissionRequired, FormView):
    template_name = "orga/review/assignment-import.html"
    permission_required = "orga.change_settings"
    form_class = ReviewAssignImportForm

    def get_form_kwargs(self):
        result = super().get_form_kwargs()
        result["event"] = self.request.event
        return result

    @transaction.atomic
    def form_valid(self, form):
        form.save()
        messages.success(self.request, _("The reviewers were assigned successfully."))
        return redirect(self.request.event.orga_urls.review_assignments)


class ReviewExport(EventPermissionRequired, FormView):
    permission_required = "orga.change_settings"
    template_name = "orga/review/export.html"
    form_class = ReviewExportForm

    def get_form_kwargs(self):
        result = super().get_form_kwargs()
        result["event"] = self.request.event
        result["user"] = self.request.user
        return result

    def form_valid(self, form):
        result = form.export_data()
        if not result:
            messages.success(self.request, _("No data to be exported"))
            return redirect(self.request.path)
        return result



import logging

from django.core.files import File
from django.core.files.temp import NamedTemporaryFile
from django.dispatch import receiver
from requests import get

from pretalx.celery_app import app
from pretalx.common.signals import periodic_task
from pretalx.person.models.user import User

logger = logging.getLogger(__name__)


@app.task()
def gravatar_cache(person_id: int):
    user = User.objects.filter(pk=person_id, get_gravatar=True).first()

    if not user:
        logger.warning(
            f"gravatar_cache() was called for user {person_id}, but "
            "user was not found or user has gravatar disabled"
        )
        return

    r = get(
        f"https://www.gravatar.com/avatar/{user.gravatar_parameter}?s=512",
        timeout=10,
    )

    logger.info(
        f"gravatar returned http {r.status_code} when getting avatar for user {user.name}"
    )

    if 400 <= r.status_code <= 499:
        # avatar not found.
        user.get_gravatar = False
        user.save()
        return
    elif r.status_code != 200:
        return

    with NamedTemporaryFile(delete=True) as tmp_img:
        for chunk in r:
            tmp_img.write(chunk)
        tmp_img.flush()

        user.avatar.save(f"{user.gravatar_parameter}.jpg", File(tmp_img))
        user.get_gravatar = False
        user.save()

        logger.info(f"set avatar for user {user.name} to {user.avatar.url}")


@receiver(periodic_task)
def refetch_gravatars(sender, **kwargs):
    users_with_gravatar = User.objects.filter(get_gravatar=True)

    for user in users_with_gravatar:
        gravatar_cache.apply_async(args=(user.pk,))



import json

import pytest


@pytest.mark.django_db
def test_api_user_endpoint(orga_client, room):
    response = orga_client.get("/api/me", follow=True)
    assert response.status_code == 200
    content = json.loads(response.content.decode())
    assert set(content.keys()) == {"name", "email", "locale", "timezone"}



from django.dispatch import receiver

from pretalx.agenda.signals import html_above_session_pages, html_below_session_pages
from pretalx.common.templatetags.rich_text import rich_text


@receiver(html_above_session_pages, dispatch_uid="html_above_sessions_settings")
def add_html_above_session_pages(sender, request, submission, **kwargs):
    text = request.event.display_settings.get("texts", {}).get(
        "agenda_session_above", ""
    )
    if text:
        return rich_text(text)
    return ""


@receiver(html_below_session_pages, dispatch_uid="html_above_sessions_settings")
def add_html_below_session_pages(sender, request, submission, **kwargs):
    text = request.event.display_settings.get("texts", {}).get(
        "agenda_session_below", ""
    )
    if text:
        return rich_text(text)
    return ""



import configparser
import importlib
from contextlib import suppress
from pathlib import Path

import pytest
from django.conf import settings
from django.dispatch import Signal

here = Path(__file__).parent
doc_dir = here / "../../../doc"
base_dir = here / "../../pretalx"

with (doc_dir / "developer/plugins/general.rst").open() as doc_file:
    plugin_docs = doc_file.read()


with (doc_dir / "administrator/commands.rst").open() as doc_file:
    command_docs = doc_file.read()


def test_documentation_includes_config_options():
    with (doc_dir / "administrator/configure.rst").open() as doc_file:
        doc_text = doc_file.read()
    config = configparser.RawConfigParser()
    config = config.read(here / "../../pretalx.example.cfg")

    for category in config:
        for key in category:
            assert key in doc_text


@pytest.mark.parametrize("app", settings.LOCAL_APPS)
def test_documentation_includes_signals(app):
    with suppress(ImportError):
        module = importlib.import_module(app + ".signals")
        for key in dir(module):
            attrib = getattr(module, key)
            if isinstance(attrib, Signal):
                assert key in plugin_docs


@pytest.mark.parametrize("app", settings.LOCAL_APPS)
def test_documentation_includes_management_commands(app):
    with suppress(ImportError):
        importlib.import_module(app + ".management.commands")
        path = base_dir / app.partition(".")[-1] / "management/commands"
        for python_file in path.glob("*.py"):
            file_name = python_file.name
            if file_name not in ["__init__.py", "makemigrations.py", "devserver.py"]:
                assert f"python -m pretalx {file_name[:-3]}``" in command_docs



import pytest
from django_scopes import scope

from pretalx.agenda.permissions import (
    is_agenda_visible,
    is_feedback_ready,
    is_speaker_viewable,
)


@pytest.mark.django_db
@pytest.mark.parametrize(
    "is_public,show_schedule,has_schedule,result",
    (
        (True, True, True, True),
        (True, True, False, False),
        (True, False, True, False),
        (False, True, True, False),
        (False, False, True, False),
    ),
)
def test_agenda_permission_is_agenda_visible(
    is_public, show_schedule, has_schedule, result, event
):
    with scope(event=event):
        event.is_public = is_public
        event.feature_flags["show_schedule"] = show_schedule
        event.save()
        if has_schedule:
            event.release_schedule("42")
        assert is_agenda_visible(None, event) is result


@pytest.mark.django_db
@pytest.mark.parametrize(
    "slot_visible,accept_feedback,result",
    (
        (True, True, True),
        (True, False, False),
        (False, True, False),
        (False, False, False),
    ),
)
def test_agenda_permission_is_feedback_ready(
    slot_visible, accept_feedback, result, slot, monkeypatch
):
    monkeypatch.setattr(
        "pretalx.agenda.permissions.is_submission_visible", lambda x, y: slot_visible
    )
    monkeypatch.setattr(
        "pretalx.submission.models.submission.Submission.does_accept_feedback",
        accept_feedback,
    )
    assert is_feedback_ready(None, slot.submission) is result


@pytest.mark.django_db
@pytest.mark.parametrize(
    "agenda_visible,result",
    (
        (True, True),
        (False, False),
    ),
)
def test_agenda_permission_is_speaker_viewable(
    agenda_visible, result, speaker, slot, schedule, monkeypatch
):
    monkeypatch.setattr(
        "pretalx.agenda.permissions.is_agenda_visible", lambda x, y: agenda_visible
    )
    with scope(event=schedule.event):
        assert is_speaker_viewable(None, speaker.profiles.first()) is result


def test_agenda_permission_is_speaker_viewable_with_wrong_params():
    assert is_speaker_viewable("user", False) is False



from django import forms
from django.conf import settings
from django.db.models import Q
from django.utils.translation import gettext_lazy as _
from django_scopes import scopes_disabled
from django_scopes.forms import SafeModelMultipleChoiceField
from i18nfield.forms import I18nModelForm

from pretalx.common.forms.fields import ImageField
from pretalx.common.mixins.forms import I18nHelpText, ReadOnlyFlag
from pretalx.event.models import Event, Organiser, Team, TeamInvite
from pretalx.orga.forms.widgets import HeaderSelect, MultipleLanguagesWidget
from pretalx.submission.models import Track


class TeamForm(ReadOnlyFlag, I18nHelpText, I18nModelForm):
    @scopes_disabled()
    def __init__(self, *args, organiser=None, instance=None, **kwargs):
        self.organiser = organiser
        super().__init__(*args, instance=instance, **kwargs)
        is_updating = instance and getattr(instance, "pk", None)
        if is_updating:
            self.fields["limit_events"].queryset = instance.organiser.events.all()
        else:
            self.fields["limit_events"].queryset = organiser.events.all()
        if is_updating and not instance.all_events and instance.limit_events.count():
            self.fields["limit_tracks"].queryset = Track.objects.filter(
                event__in=instance.limit_events.all()
            )
        else:
            self.fields["limit_tracks"].queryset = Track.objects.filter(
                event__organiser=organiser
            ).order_by("-event__date_from", "name")

    @scopes_disabled()
    def save(self, *args, **kwargs):
        self.instance.organiser = self.organiser
        return super().save(*args, **kwargs)

    def clean(self):
        data = super().clean()
        all_events = data.get("all_events")
        limit_events = data.get("limit_events")
        if not all_events and not limit_events:
            error = forms.ValidationError(
                _(
                    "Please either pick some events for this team, or grant access to all your events!"
                )
            )
            self.add_error("limit_events", error)
        permissions = (
            "can_create_events",
            "can_change_teams",
            "can_change_organiser_settings",
            "can_change_event_settings",
            "can_change_submissions",
            "is_reviewer",
        )
        if not any(data.get(permission) for permission in permissions):
            error = forms.ValidationError(
                _("Please pick at least one permission for this team!")
            )
            self.add_error(None, error)
        return data

    class Meta:
        model = Team
        fields = [
            "name",
            "all_events",
            "limit_events",
            "can_create_events",
            "can_change_teams",
            "can_change_organiser_settings",
            "can_change_event_settings",
            "can_change_submissions",
            "is_reviewer",
            "force_hide_speaker_names",
            "limit_tracks",
        ]
        widgets = {
            "limit_events": forms.SelectMultiple(attrs={"class": "select2"}),
            "limit_tracks": forms.SelectMultiple(attrs={"class": "select2"}),
        }
        field_classes = {
            "limit_tracks": SafeModelMultipleChoiceField,
        }


class TeamInviteForm(ReadOnlyFlag, forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["email"].required = True

    class Meta:
        model = TeamInvite
        fields = ("email",)


class OrganiserForm(ReadOnlyFlag, I18nHelpText, I18nModelForm):
    def __init__(self, *args, **kwargs):
        kwargs["locales"] = "en"
        super().__init__(*args, **kwargs)

        if kwargs.get("instance"):
            self.fields["slug"].disabled = True

    class Meta:
        model = Organiser
        fields = ("name", "slug")


class EventWizardInitialForm(forms.Form):
    locales = forms.MultipleChoiceField(
        choices=settings.LANGUAGES,
        label=_("Use languages"),
        help_text=_("Choose all languages that your event should be available in."),
        widget=MultipleLanguagesWidget,
    )

    def __init__(self, *args, user=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["organiser"] = forms.ModelChoiceField(
            label=_("Organiser"),
            queryset=Organiser.objects.filter(
                id__in=user.teams.filter(can_create_events=True).values_list(
                    "organiser", flat=True
                )
            )
            if not user.is_administrator
            else Organiser.objects.all(),
            widget=forms.RadioSelect,
            empty_label=None,
            required=True,
            help_text=_(
                "The organiser running the event can copy settings from previous events and share team permissions across all or multiple events."
            ),
        )
        self.fields["organiser"].initial = self.fields["organiser"].queryset.first()


class EventWizardBasicsForm(I18nHelpText, I18nModelForm):
    def __init__(self, *args, user=None, locales=None, organiser=None, **kwargs):
        self.locales = locales or []
        super().__init__(*args, **kwargs, locales=locales)
        self.fields["locale"].choices = [
            (a, b) for a, b in settings.LANGUAGES if a in locales
        ]
        self.fields["slug"].help_text = (
            str(
                _(
                    "This is the address your event will be available at. "
                    "Should be short, only contain lowercase letters and numbers, and must be unique. "
                    "We recommend some kind of abbreviation with less than 10 characters that can be easily remembered."
                )
            )
            + " <strong>"
            + str(_("You cannot change the slug later on!"))
            + "</strong>"
        )

    def clean_slug(self):
        slug = self.cleaned_data["slug"]
        qs = Event.objects.all()
        if qs.filter(slug__iexact=slug).exists():
            raise forms.ValidationError(
                _(
                    "This short name is already taken, please choose another one (or ask the owner of that event to add you to their team)."
                )
            )

        return slug.lower()

    class Meta:
        model = Event
        fields = ("name", "slug", "timezone", "email", "locale")


class EventWizardTimelineForm(forms.ModelForm):
    deadline = forms.DateTimeField(
        required=False,
        help_text=_(
            "The default deadline for your Call for Papers. You can assign additional deadlines to individual session types, which will take precedence over this deadline."
        ),
    )

    def __init__(self, *args, user=None, locales=None, organiser=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["deadline"].widget.attrs["class"] = "datetimepickerfield"

    def clean(self):
        data = super().clean()
        date_from = data.get("date_from")
        date_to = data.get("date_to")
        if date_from and date_to and date_from > date_to:
            error = forms.ValidationError(
                _("The event end cannot be before the start.")
            )
            self.add_error("date_from", error)
        return data

    class Meta:
        model = Event
        fields = ("date_from", "date_to")
        widgets = {
            "date_from": forms.DateInput(attrs={"class": "datepickerfield"}),
            "date_to": forms.DateInput(attrs={"class": "datepickerfield"}),
        }


class EventWizardDisplayForm(forms.Form):
    primary_color = forms.CharField(
        max_length=7,
        label=_("Main event colour"),
        help_text=_(
            "Provide a hex value like #00ff00 if you want to style pretalx in your event's colour scheme."
        ),
        required=False,
    )
    header_pattern = forms.ChoiceField(
        label=_("Frontpage header pattern"),
        help_text=_(
            'Choose how the frontpage header banner will be styled. Pattern source: <a href="http://www.heropatterns.com/">heropatterns.com</a>, CC BY 4.0.'
        ),
        choices=(
            ("", _("Plain")),
            ("pcb", _("Circuits")),
            ("bubbles", _("Circles")),
            ("signal", _("Signal")),
            ("topo", _("Topography")),
            ("graph", _("Graph Paper")),
        ),
        required=False,
        widget=HeaderSelect,
    )

    def __init__(self, *args, user=None, locales=None, organiser=None, **kwargs):
        super().__init__(*args, **kwargs)
        logo = Event._meta.get_field("logo")
        self.fields["logo"] = ImageField(
            required=False, label=logo.verbose_name, help_text=logo.help_text
        )
        self.fields["primary_color"].widget.attrs["class"] = "colorpickerfield"


class EventWizardCopyForm(forms.Form):
    @staticmethod
    def copy_from_queryset(user):
        return Event.objects.filter(
            Q(
                organiser_id__in=user.teams.filter(
                    all_events=True, can_change_event_settings=True
                ).values_list("organiser", flat=True)
            )
            | Q(
                id__in=user.teams.filter(can_change_event_settings=True).values_list(
                    "limit_events__id", flat=True
                )
            )
        )

    def __init__(self, *args, user=None, locales=None, organiser=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["copy_from_event"] = forms.ModelChoiceField(
            label=_("Copy configuration from"),
            queryset=EventWizardCopyForm.copy_from_queryset(user),
            widget=forms.RadioSelect,
            empty_label=_("Do not copy"),
            required=False,
        )



import pytest
from django_scopes import scopes_disabled

from pretalx.submission.utils import move_submission


@pytest.mark.django_db
@scopes_disabled()
def test_simple_submission_move(other_event, submission):
    assert submission.event != other_event
    assert not submission.speakers.first().profiles.filter(event=other_event)

    move_submission(submission.code, other_event.slug)
    submission.refresh_from_db()

    assert submission.event == other_event
    assert submission.speakers.first().profiles.filter(event=other_event)


@pytest.mark.django_db
@scopes_disabled()
def test_simple_submission_move_with_questions(
    other_event, submission, answer, speaker_answer
):
    # set up matching questions
    answer.question.id = None
    answer.question.event = other_event
    answer.question.save()
    speaker_answer.question.id = None
    speaker_answer.question.event = other_event
    speaker_answer.question.save()
    assert other_event.questions.all().count() == 2

    assert submission.event != other_event
    assert not submission.speakers.first().profiles.filter(event=other_event)

    move_submission(submission.code, other_event.slug)
    submission.refresh_from_db()

    assert submission.event == other_event
    assert submission.speakers.first().profiles.filter(event=other_event)
    assert submission.answers.all().filter(question__event=other_event)
    assert submission.speakers.first().answers.all().filter(question__event=other_event)



import json

import pytest
from django_scopes import scope

from pretalx.submission.models import Answer


@pytest.mark.django_db
@pytest.mark.parametrize("is_public", (True, False))
def test_answers_not_visible_by_default(client, answer, schedule, is_public):
    with scope(event=answer.event):
        answer.question.is_public = is_public
        answer.question.save()
    response = client.get(answer.event.api_urls.answers, follow=True)
    content = json.loads(response.content.decode())
    assert response.status_code == 200
    assert bool(len(content["results"])) is is_public


@pytest.mark.django_db
def test_organizer_can_see_answer(orga_client, answer):
    response = orga_client.get(answer.event.api_urls.answers, follow=True)
    content = json.loads(response.content.decode())
    assert response.status_code == 200
    assert len(content["results"]) == 1
    assert content["results"][0]["id"] == answer.id


@pytest.mark.django_db
@pytest.mark.parametrize("is_visible", (True, False))
def test_answers_not_visible_by_default_to_reviewers(review_client, answer, is_visible):
    with scope(event=answer.event):
        answer.question.is_visible_to_reviewers = is_visible
        answer.question.save()
    response = review_client.get(answer.question.event.api_urls.answers, follow=True)
    content = json.loads(response.content.decode())
    assert response.status_code == 200
    assert bool(len(content["results"])) is is_visible


@pytest.mark.django_db
def test_organizer_can_create_answer(event, orga_client, question, submission, speaker):
    with scope(event=event):
        count = Answer.objects.filter(question__event=event).count()
    response = orga_client.post(
        event.api_urls.answers,
        {
            "question": question.id,
            "submission": submission.code,
            "person": speaker.code,
            "answer": "Tralalalala",
        },
    )
    assert response.status_code == 201, response.content.decode()
    with scope(event=event):
        assert Answer.objects.filter(question__event=event).count() == count + 1
        answer = Answer.objects.filter(question__event=event).first()
        assert answer.answer == "Tralalalala"


@pytest.mark.django_db
def test_duplicate_answer_updates_existing_answer(
    event, orga_client, question, submission, speaker, answer
):
    with scope(event=event):
        count = Answer.objects.filter(question__event=event).count()
        response = orga_client.post(
            event.api_urls.answers,
            {
                "question": answer.question_id,
                "submission": answer.submission.code,
                "person": "",
                "answer": "Tralalalala",
            },
        )
    assert response.status_code == 201, response.content.decode()
    with scope(event=event):
        assert Answer.objects.filter(question__event=event).count() == count
        answer = Answer.objects.filter(question__event=event).first()
        assert answer.answer == "Tralalalala"


@pytest.mark.django_db
def test_organizer_can_edit_answers(event, orga_client, answer):
    response = orga_client.patch(
        event.api_urls.answers + f"{answer.pk}/",
        {"answer": "ohno.png"},
        content_type="application/json",
    )
    assert response.status_code == 200, response.content.decode()
    with scope(event=event):
        answer.refresh_from_db()
        assert answer.answer == "ohno.png"


@pytest.mark.django_db
def test_reviewer_cannot_create_answer(
    event, review_client, question, submission, speaker
):
    with scope(event=event):
        count = Answer.objects.filter(question__event=event).count()
    response = review_client.post(
        event.api_urls.answers,
        {
            "question": question.id,
            "submission": submission.code,
            "person": speaker.code,
            "answer": "Tralalalala",
        },
    )
    assert response.status_code == 403, response.content.decode()
    with scope(event=event):
        assert Answer.objects.filter(question__event=event).count() == count


@pytest.mark.django_db
def test_reviewer_cannot_edit_answer(event, review_client, answer):
    response = review_client.patch(
        event.api_urls.answers + f"{answer.pk}/",
        {"answer": "ohno.png"},
        content_type="application/json",
    )
    assert response.status_code == 403, response.content.decode()
    with scope(event=event):
        answer.refresh_from_db()
        assert answer.answer != "ohno.png"



import json

import pytest
from django_scopes import scope


@pytest.mark.django_db
def test_user_can_see_schedule(client, slot, event):
    with scope(event=event):
        assert slot.submission.event.schedules.count() == 2
    response = client.get(slot.submission.event.api_urls.schedules, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 1


@pytest.mark.django_db
def test_user_cannot_see_wip_schedule(client, slot, event):
    with scope(event=event):
        assert slot.submission.event.schedules.count() == 2
    response = client.get(slot.submission.event.api_urls.schedules + "wip", follow=True)
    json.loads(response.content.decode())
    assert response.status_code == 404


@pytest.mark.django_db
def test_user_cannot_see_schedule_if_not_public(client, slot, event):
    slot.submission.event.feature_flags["show_schedule"] = False
    slot.submission.event.save()
    with scope(event=event):
        assert slot.submission.event.schedules.count() == 2
    response = client.get(slot.submission.event.api_urls.schedules, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 0


@pytest.mark.django_db
def test_orga_can_see_schedule(orga_client, slot, event):
    with scope(event=event):
        assert slot.submission.event.schedules.count() == 2
    response = orga_client.get(slot.submission.event.api_urls.schedules, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 2


@pytest.mark.django_db
def test_orga_can_see_wip_schedule(orga_client, slot, event):
    with scope(event=event):
        assert slot.submission.event.schedules.count() == 2
    response = orga_client.get(
        slot.submission.event.api_urls.schedules + "wip", follow=True
    )
    json.loads(response.content.decode())
    assert response.status_code == 200


@pytest.mark.django_db
def test_orga_can_see_current_schedule(orga_client, slot, event):
    with scope(event=event):
        assert slot.submission.event.schedules.count() == 2
    response = orga_client.get(
        slot.submission.event.api_urls.schedules + "latest", follow=True
    )
    json.loads(response.content.decode())
    assert response.status_code == 200
    with scope(event=event):
        assert slot.submission.title in response.content.decode()


@pytest.mark.django_db
def test_orga_cannot_see_schedule_even_if_not_public(orga_client, slot, event):
    slot.submission.event.feature_flags["show_schedule"] = False
    slot.submission.event.save()
    with scope(event=event):
        assert slot.submission.event.schedules.count() == 2
    response = orga_client.get(slot.submission.event.api_urls.schedules, follow=True)
    content = json.loads(response.content.decode())

    assert response.status_code == 200
    assert content["count"] == 2



# Generated by Django 4.0 on 2023-06-03 21:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("person", "0025_drop_unused_speakerinformation_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="timezone",
            field=models.CharField(default="UTC", max_length=32),
        ),
    ]



import pytest
from django.forms import ValidationError

from pretalx.common.forms.utils import validate_field_length


@pytest.mark.parametrize(
    "value,min_length,max_length,count_in,valid",
    (
        ("word word word", None, None, "chars", True),
        ("word word word", None, None, "words", True),
        ("word word word", 1, None, "chars", True),
        ("word word word", 1, None, "words", True),
        ("word word word", None, 300, "chars", True),
        ("word word word", None, 3, "words", True),
        ("word word word", 1, 300, "chars", True),
        ("word word word", 1, 3, "words", True),
        ("word word word", 100, None, "chars", False),
        ("word word word", 4, None, "words", False),
        ("word word word", None, 3, "chars", False),
        ("word word word", None, 2, "words", False),
        ("word word word", 10, 11, "chars", False),
        ("word word word", 2, 2, "words", False),
    ),
)
def test_validate_field_length(value, min_length, max_length, count_in, valid):
    if valid:
        assert (
            validate_field_length(
                value=value,
                min_length=min_length,
                max_length=max_length,
                count_in=count_in,
            )
            is None
        )
    else:
        with pytest.raises(ValidationError):
            validate_field_length(
                value=value,
                min_length=min_length,
                max_length=max_length,
                count_in=count_in,
            )



import pytest
from django_scopes import scope

from pretalx.common.tasks import generate_widget_css, generate_widget_js, regenerate_css


@pytest.mark.django_db
def test_regenerate_css_with_new_checksum(event):
    with scope(event=event):
        event.primary_color = "#ff0000"
        event.save()

        regenerate_css(event.pk)
        regenerate_css(event.pk)  # Second time will match the checksum


@pytest.mark.django_db
def test_generate_widget_css(event):
    with scope(event=event):
        generate_widget_css(event, save=False)
        generate_widget_css(event, save=True)
        generate_widget_css(event, save=True)


@pytest.mark.django_db
def test_generate_widget_js(event):
    with scope(event=event):
        generate_widget_js(event, "en", save=False)
        generate_widget_js(event, "en", save=True)
        generate_widget_js(event, "en", save=True)
